@@ -24,7 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef _BSD
+#if defined(_BSD) || defined(__APPLE__)
 #include <sys/param.h>
 #else
 #include <values.h>
@@ -235,7 +235,7 @@ void send_echo_udp(int seq, int size, int ttl, uint32_t dst_ip)
 	    ui->ui_dst = oui->ui_dst;
 	    ui->ui_pr = oui->ui_pr;
 
-#if defined SUN || defined _BSD
+#if defined SUN || defined _BSD || defined __APPLE__
 	    outudp->uh_dport = htons(port + seq);
 	    outudp->uh_ulen = htons((u_short)(size - (sizeof(*outip))));
 	    ui->ui_len = outudp->uh_ulen;
@@ -334,7 +334,7 @@ void set_udp_header()
 	outip->ip_p = IPPROTO_UDP;
 	outudp = (struct udphdr *)outp;
 	// outudp->source = htons(ident);
-#if defined SUN || defined _BSD
+#if defined SUN || defined _BSD || defined __APPLE__
 	outudp->uh_sport = htons(udp_port);
 #else
 	outudp->source = htons(udp_port);
@@ -413,6 +413,12 @@ void dump_route()
 	printf("%s %d\n", info_line1, delay_num);
 	printf("%s", info_line2);
 
+	/* no valid reponse */
+	if (ip_path_len == 0 && !ip_path[0].ip) {
+	    printf("# no hop reponses\n\n");
+	    return;
+	}
+
 	printf("\n");
 	j = 0;
 	for (i=0; i<=ip_path_len; i++) {
@@ -437,7 +443,7 @@ void dump_route()
 	    if (on_line_processing) {
 		if (i == node[j].index) {
 		    printf(" %6d", node[j].gap1);
-		    if (selected[i]) 
+		    if (selected[j]) 
 			printf(" %c", selected[j][1]);
 		    else
 			printf(" %c", '.');
@@ -532,10 +538,13 @@ void recv_pkts(int last)
 		    if (ntohl(icmp->icmp_mask) != MAGIC_NUM)
 			continue;
 
-		    memcpy(rtt_rec[rtt_i], buffer, len);
-		    rtt_arr_time[rtt_i] = get_time();
-		    pre_time = rtt_arr_time[rtt_i];
-		    rtt_i ++;
+		    /* if "-c" is specified, rtt_i could exceed MAX_RTT_NUM */
+		    if (rtt_i < MAX_RTT_NUM && len == 60) {
+			memcpy(rtt_rec[rtt_i], buffer, len);
+			rtt_arr_time[rtt_i] = get_time();
+			pre_time = rtt_arr_time[rtt_i];
+			rtt_i ++;
+		    }
 		    continue;
 		}
 
@@ -721,7 +730,7 @@ void store()
 
 		if (!icmp_probing) {
 		    udphdr = (struct udphdr *)((char *)icmp + 8 + sizeof(struct ip));
-#if defined SUN || defined _BSD
+#if defined SUN || defined _BSD || defined __APPLE__
 		    j = ntohs(udphdr->uh_dport) - port;
 #else
 		    j = ntohs(udphdr->dest) - port;
